import dash_html_components as html
from . import WebvizContainer


class ExampleDataDownload(WebvizContainer):

    def __init__(self, app, title: str):
        self.title = title
        self.set_callbacks(app)

    @property
    def layout(self):
        return html.H1(self.title)

    def set_callbacks(self, app):
        @app.callback(self.container_data_output,
                      [self.container_data_requested])
        def _user_download_data(data_requested):
            return WebvizContainer.container_data_compress(
                [{'filename': 'some_file.txt',
                  'content': 'Some download data'}]
            ) if data_requested else ''



import dash
from webviz_config.common_cache import cache
from webviz_config.containers import _example_container


def test_example_container(dash_duo):

    app = dash.Dash(__name__)
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)
    title = 'Example'
    page = _example_container.ExampleContainer(app, title)
    app.layout = page.layout
    dash_duo.start_server(app)
    btn = dash_duo.find_element(f'#{page.button_id}')
    assert 'Submit' == btn.text
    text = dash_duo.find_element(f'#{page.div_id}')
    assert 'Button has been pressed 0 times.' == text.text
    btn.click()
    dash_duo.wait_for_contains_text(f'#{page.div_id}',
                                    'Button has been pressed 1 times',
                                    timeout=2)
    assert 'Button has been pressed 1 times.' == text.text



from uuid import uuid4
from pathlib import Path
import pandas as pd
import dash_table
from . import WebvizContainer
from ..webviz_store import webvizstore
from ..common_cache import cache


class DataTable(WebvizContainer):
    '''### Data table

This container adds a table to the webviz instance, using tabular data from
a provided csv file. If feature is requested, the data could also come from
a database.

* `csv_file`: Path to the csv file containing the tabular data. Either absolute
              path or relative to the configuration file.
* `sorting`: If `True`, the table can be sorted interactively based
             on data in the individual columns.
* `filtering`: If `True`, the table can be filtered based on values in the
               individual columns.
* `pagination`: If `True`, only a subset of the table is displayed at once.
                Different subsets can be viewed from 'previous/next' buttons
'''

    def __init__(self, csv_file: Path, sorting: bool = True,
                 filtering: bool = True, pagination: bool = True):

        self.csv_file = csv_file
        self.df = get_data(self.csv_file)
        self.sorting = sorting
        self.filtering = filtering
        self.pagination = pagination
        self.data_table_id = 'data-table-{}'.format(uuid4())

    def add_webvizstore(self):
        return [(get_data, [{'csv_file': self.csv_file}])]

    @property
    def layout(self):
        return dash_table.DataTable(
            id=self.data_table_id,
            columns=[{'name': i, 'id': i} for i in self.df.columns],
            data=self.df.to_dict('records'),
            sort_action='native' if self.sorting else 'none',
            filter_action='native' if self.filtering else 'none',
            page_action='native' if self.pagination else 'none'
        )


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def get_data(csv_file) -> pd.DataFrame:
    return pd.read_csv(csv_file)



import argparse
from ._build_webviz import build_webviz
from ._certificate import create_ca


def main():

    parser = argparse.ArgumentParser(prog=('Creates a Webviz Dash app '
                                           'from a configuration setup'))

    subparsers = parser.add_subparsers(help='The options available. '
                                            'Type e.g. "webviz build --help" '
                                            'to get help on that particular '
                                            'option.')

    # Add "build" argument parser:

    parser_build = subparsers.add_parser('build', help='Build a Webviz '
                                                       'Dash App')

    parser_build.add_argument('yaml_file', type=str,
                              help='Path to YAML configuration file')
    parser_build.add_argument('--portable', type=str, default=None,
                              metavar='OUTPUTFOLDER',
                              help='A portable webviz instance will created '
                                   'and saved to the given folder.')
    parser_build.add_argument('--not-only-localhost', action='store_true',
                              help='Front-end accesible on internal network')
    parser_build.add_argument('--theme', type=str, default='default',
                              help='Which installed theme to use.')

    def check_port(port):
        port = int(port)
        if port <= 1024:
            argparse.ArgumentTypeError('Port must be greater than 1024')
        return port

    parser_build.add_argument('--port', type=check_port, default=8050,
                              help='Port to use for the server.')

    parser_build.set_defaults(func=build_webviz)

    # Add "certificate" parser:

    parser_cert = subparsers.add_parser('certificate', help='Create a root '
                                                            'certificate '
                                                            'for https')

    parser_cert.add_argument('--force', action='store_true',
                             help='Overwrite webviz root https '
                                  'certificate if it already exist')

    parser_cert.set_defaults(func=create_ca)

    # Do the argument parsing:

    args = parser.parse_args()

    args.func(args)



from pathlib import Path
import dash_html_components as html
from . import WebvizContainer
from ..webviz_assets import webviz_assets


class EmbedPdf(WebvizContainer):
    '''### Embed PDF file

This container embeds a given PDF file into the page.

* `pdf_file`: Path to the PDF file to include. Either absolute path or
  relative to the configuration file.
* `height`: Height of the PDF object (in percent of viewport height).
* `width`: Width of the PDF object (in percent of available space).

_Note_: Webviz does not scan your PDF for malicious code.
Make sure it comes from a trusted source.
'''

    def __init__(self, pdf_file: Path, height: int = 80, width: int = 100):
        self.pdf_url = webviz_assets.add(pdf_file)
        self.height = height
        self.width = width

    @property
    def layout(self):

        style = {
                 'height': f'{self.height}vh',
                 'width': f'{self.width}%'
                }

        return html.Embed(src=self.pdf_url,
                          style=style,
                          type='application/pdf')



import sys
import subprocess


def test_portable(dash_duo, tmp_path):
    # Build a portable webviz from config file
    appdir = tmp_path / 'app'
    subprocess.call(['webviz', 'build', 'basic_example.yaml',
                     '--portable', appdir], cwd='examples')
    # Remove Talisman
    fn = appdir / 'webviz_app.py'
    with open(fn, "r") as f:
        lines = f.readlines()
    with open(fn, "w") as f:
        for line in lines:
            if not line.strip("\n").startswith("Talisman"):
                f.write(line)
    # Import generated app
    sys.path.append(str(appdir))
    from webviz_app import app
    # Start and test app
    dash_duo.start_server(app)
    for page in ['markdown_example', 'table_example', 'pdf_example',
                 'syntax_highlighting_example', 'plot_a_table', 'last_page']:
        dash_duo.wait_for_element(f'#{page}').click()
    assert dash_duo.get_logs() == [], 'browser console should contain no error'



from uuid import uuid4
from pathlib import Path
import dash_core_components as dcc
from . import WebvizContainer
from ..webviz_store import webvizstore


class SyntaxHighlighter(WebvizContainer):
    '''### Syntax highlighter

This container adds support for syntax highlighting of code. Language is
automatically detected.

* `filename`: Path to a file containing the code to highlight.
* `dark_theme`: If `True`, the code is shown with a dark theme. Default is
                `False` giving a light theme.
'''

    def __init__(self, filename: Path, dark_theme: bool = False):

        self.filename = filename
        self.config = {'theme': 'dark'} if dark_theme else {'theme': 'light'}

    def add_webvizstore(self):
        return [(get_path, [{'path': self.filename}])]

    @property
    def layout(self):
        return dcc.Markdown(f'```{get_path(self.filename).read_text()}```',
                            highlight_config=self.config)


@webvizstore
def get_path(path) -> Path:
    return path



import pkg_resources

from ._theme_class import WebvizConfigTheme
from ._default_theme import default_theme

installed_themes = {default_theme.theme_name: default_theme}

__all__ = ['WebvizConfigTheme',
           'installed_themes']

for entry_point in pkg_resources.iter_entry_points('webviz_config_themes'):
    theme = entry_point.load()

    globals()[entry_point.name] = theme
    __all__.append(entry_point.name)

    if isinstance(theme, WebvizConfigTheme):
        installed_themes[theme.theme_name] = theme



from uuid import uuid4
import dash_html_components as html
from dash.dependencies import Input, Output
from . import WebvizContainer


class ExampleContainer(WebvizContainer):

    def __init__(self, app, title: str):
        self.title = title

        self.button_id = 'submit-button-{}'.format(uuid4())
        self.div_id = 'output-state-{}'.format(uuid4())

        self.set_callbacks(app)

    @property
    def layout(self):
        return html.Div([
                         html.H1(self.title),
                         html.Button(id=self.button_id, n_clicks=0,
                                     children='Submit'),
                         html.Div(id=self.div_id)
                        ])

    def set_callbacks(self, app):
        @app.callback(Output(self.div_id, 'children'),
                      [Input(self.button_id, 'n_clicks')])
        def _update_output(n_clicks):
            return 'Button has been pressed {} times.'.format(n_clicks)



'''### _Basic containers_

These are the basic Webviz configuration containers, distributed through
the utility itself.
'''

import pkg_resources

from ._container_class import WebvizContainer

from ._example_container import ExampleContainer
from ._example_data_download import ExampleDataDownload
from ._example_assets import ExampleAssets
from ._example_portable import ExamplePortable
from ._banner_image import BannerImage
from ._data_table import DataTable
from ._syntax_highlighter import SyntaxHighlighter
from ._table_plotter import TablePlotter
from ._embed_pdf import EmbedPdf
from ._markdown import Markdown

__all__ = ['WebvizContainer',
           'ExampleContainer',
           'ExampleAssets',
           'ExamplePortable',
           'BannerImage',
           'DataTable',
           'SyntaxHighlighter',
           'TablePlotter',
           'EmbedPdf',
           'Markdown']

for entry_point in pkg_resources.iter_entry_points('webviz_config_containers'):
    globals()[entry_point.name] = entry_point.load()

    __all__.append(entry_point.name)



import os
import re
import sys
import socket
import datetime
import getpass
from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import rsa


NAME = x509.Name([
          x509.NameAttribute(NameOID.COUNTRY_NAME, 'NO'),
          x509.NameAttribute(NameOID.LOCALITY_NAME, 'Trondheim'),
          x509.NameAttribute(NameOID.ORGANIZATION_NAME, 'Webviz'),
          x509.NameAttribute(NameOID.COMMON_NAME, f'{getpass.getuser()}'),
          x509.NameAttribute(NameOID.ORGANIZATIONAL_UNIT_NAME, 'Webviz'),
       ])

CA_KEY_FILENAME = 'ca.key'
CA_CRT_FILENAME = 'ca.crt'

SERVER_KEY_FILENAME = 'server.key'
SERVER_CRT_FILENAME = 'server.crt'

DNS_NAME = re.sub('[0-9]+', '*', socket.getfqdn())


def user_data_dir():
    '''Returns platform specific path to store user application data
    '''

    if sys.platform == "win32":
        return os.path.normpath(os.path.expanduser('~/Application Data/'
                                                   'webviz_cert'))
    elif sys.platform == 'darwin':
        return os.path.expanduser('~/Library/Application Support/webviz')
    else:
        return os.path.expanduser("~/.local/share/webviz")


def create_key(key_path):

    key = rsa.generate_private_key(public_exponent=65537,
                                   key_size=2048,
                                   backend=default_backend())

    with open(key_path, "wb") as fh:
        fh.write(key.private_bytes(
               encoding=serialization.Encoding.PEM,
               format=serialization.PrivateFormat.TraditionalOpenSSL,
               encryption_algorithm=serialization.NoEncryption()
              ))

    return key


def certificate_template(subject, issuer, public_key, ca=False):

    if ca:
        not_valid_after = datetime.datetime.utcnow() + \
                          datetime.timedelta(days=365)

    else:  # shorter valid length for on-the-fly certificates
        not_valid_after = datetime.datetime.utcnow() + \
                          datetime.timedelta(days=7)

    return x509.CertificateBuilder()\
               .subject_name(subject)\
               .issuer_name(issuer)\
               .public_key(public_key)\
               .serial_number(x509.random_serial_number())\
               .not_valid_before(datetime.datetime.utcnow())\
               .not_valid_after(not_valid_after)\
               .add_extension(x509.SubjectAlternativeName(
                                      [
                                       x509.DNSName('localhost'),
                                       x509.DNSName(DNS_NAME)
                                      ]),
                              critical=True)\
               .add_extension(x509.BasicConstraints(ca=ca, path_length=None),
                              critical=True)


def create_ca(args):

    directory = user_data_dir()

    os.makedirs(directory, exist_ok=True)

    ca_key_path = os.path.join(directory, CA_KEY_FILENAME)
    ca_crt_path = os.path.join(directory, CA_CRT_FILENAME)

    if not args.force and os.path.isfile(ca_crt_path):
        raise OSError(f'The file {ca_crt_path} already exists. Add the '
                      'command line flag --force if you want to overwrite')

    key = create_key(ca_key_path)

    subject = issuer = NAME

    cert = certificate_template(subject, issuer, key.public_key(), ca=True)\
        .sign(key, hashes.SHA256(), default_backend())

    with open(ca_crt_path, "wb") as fh:
        fh.write(cert.public_bytes(serialization.Encoding.PEM))

    sha1 = "-".join(re.findall('.{8,8}',
                               cert.fingerprint(hashes.SHA1()).hex())).upper()

    print(f'''\n\033[1m\033[94m
 Created CA key and certificate files (both saved in {directory}).
 Keep the key file ({CA_KEY_FILENAME}) private. The certificate file
 ({CA_CRT_FILENAME}) is not sensitive, and you can import it in
 your browser(s).

 To install it in Chrome:

    - Open Chrome and go to chrome://settings/privacy
    - Select "Manage certificates"
    - Under the tab "Trusted Root Certificatation Authorities", click "Import"
    - Go to {directory} and select the created certificate ({CA_CRT_FILENAME}).
    - Click "Next" and select "Place all certificates in the following store"
    - Click "Next" and then "Finished"
    - If a dialog box appears, you can verify that the displayed thumbprint is
      the same as this one:
       {sha1}
    - Restart Chrome

 When done, you do not have to rerun "webviz certificate" or do this procedure
 before the certificate expiry date has passed. The certificate is only valid
 for localhost and {DNS_NAME}.''')


def create_certificate(directory):

    ca_directory = user_data_dir()
    ca_key_path = os.path.join(ca_directory, CA_KEY_FILENAME)
    ca_crt_path = os.path.join(ca_directory, CA_CRT_FILENAME)

    server_key_path = os.path.join(directory, SERVER_KEY_FILENAME)
    server_crt_path = os.path.join(directory, SERVER_CRT_FILENAME)

    if not os.path.isfile(ca_key_path) or not os.path.isfile(ca_crt_path):
        raise RuntimeError('Could not find CA key and certificate. Please '
                           'run the command "webviz certificate" and '
                           'try again')

    with open(ca_key_path, 'rb') as fh:
        ca_key = serialization.load_pem_private_key(
            data=fh.read(),
            password=None,
            backend=default_backend()
        )

    with open(ca_crt_path, 'rb') as f:
        ca_crt = x509.load_pem_x509_certificate(data=f.read(),
                                                backend=default_backend())

    server_key = create_key(server_key_path)

    crt = certificate_template(NAME, ca_crt.subject, server_key.public_key())\
        .add_extension(critical=True,
                       extension=x509.KeyUsage(
                                           digital_signature=True,
                                           key_encipherment=True,
                                           content_commitment=True,
                                           data_encipherment=False,
                                           key_agreement=False,
                                           encipher_only=False,
                                           decipher_only=False,
                                           key_cert_sign=False,
                                           crl_sign=False))\
        .add_extension(critical=False,
                       extension=x509.AuthorityKeyIdentifier
                       .from_issuer_public_key(ca_key.public_key()))\
        .sign(private_key=ca_key,
              algorithm=hashes.SHA256(),
              backend=default_backend())

    with open(server_crt_path, 'wb') as f:
        f.write(crt.public_bytes(encoding=serialization.Encoding.PEM))



import numpy as np
import pandas as pd
from collections import OrderedDict
from pathlib import Path
from uuid import uuid4
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output
import plotly.express as px
from . import WebvizContainer
from ..webviz_store import webvizstore
from ..common_cache import cache


class TablePlotter(WebvizContainer):
    '''### TablePlotter

This container adds a plotter to the webviz instance, using tabular data from
a provided csv file. If feature is requested, the data could also come from
a database.

* `csv_file`: Path to the csv file containing the tabular data. Either absolute
              path or relative to the configuration file.
* `plot_options`: A dictionary of plot options to initialize the plot with
* `lock`: If `True`, only the plot is shown, all dropdowns for changing
          plot options are hidden.
'''

    def __init__(self, app, csv_file: Path, plot_options: dict = None,
                 lock: bool = False):

        self.plot_options = plot_options if plot_options else {}
        self.graph_id = f'graph-id{uuid4()}'
        self.lock = lock
        self.csv_file = csv_file
        self.data = get_data(self.csv_file)
        self.columns = list(self.data.columns)
        self.numeric_columns = list(
            self.data.select_dtypes(include=[np.number]).columns)
        self.selector_row = f'selector-row{uuid4()}'
        self.plot_option_id = f'plot-option{uuid4()}'
        self.set_callbacks(app)

    def add_webvizstore(self):
        return [(get_data, [{'csv_file': self.csv_file}])]

    @property
    def plots(self):
        '''A list of available plots and their options'''
        return {
            'scatter': ['x', 'y', 'size', 'color', 'facet_col'],
            'histogram': ['x', 'y', 'color', 'facet_col', 'barmode',
                          'barnorm', 'histnorm'],
            'bar': ['x', 'y', 'color', 'facet_col'],
            'scatter_3d': ['x', 'y', 'z', 'size', 'color'],
            'line': ['x', 'y', 'color', 'line_group', 'facet_col'],
            'line_3d': ['x', 'y', 'z', 'color'],
            'box': ['x', 'y', 'color', 'facet_col'],
            'violin': ['x', 'y', 'color', 'facet_col'],
            'scatter_matrix': ['dimensions', 'size', 'color'],
            'parallel_coordinates': ['dimensions'],
            'parallel_categories': ['dimensions'],
            'density_contour': ['x', 'y', 'color', 'facet_col'],
        }

    @property
    def plot_args(self):
        '''A list of possible plot options and their default values'''
        return OrderedDict(
            {
                'x': {
                    'options': self.columns,
                    'value': self.plot_options.get('x', self.columns[0]),
                    'multi': False
                },
                'y': {
                    'options': self.columns,
                    'value': self.plot_options.get('y', self.columns[0]),
                    'multi': False
                },
                'z': {
                    'options': self.columns,
                    'value': self.plot_options.get('z', self.columns[0]),
                    'multi': False
                },
                'size': {
                    'options': self.numeric_columns,
                    'value': self.plot_options.get('size', None),
                    'multi': False
                },
                'color': {
                    'options': self.columns,
                    'value': self.plot_options.get('color', None),
                    'multi': False
                },
                'facet_col': {
                    'options': self.columns,
                    'value': self.plot_options.get('facet_col', None),
                    'multi': False
                },
                'line_group': {
                    'options': self.columns,
                    'value': self.plot_options.get('line_group', None),
                    'multi': False
                },
                'barmode': {
                    'options': ['stack', 'group', 'overlay', 'relative'],
                    'value': self.plot_options.get('barmode', 'stack'),
                    'multi': False
                },
                'barnorm': {
                    'options': ['fraction', 'percent'],
                    'value': self.plot_options.get('barnorm', None),
                    'multi': False
                },
                'histnorm': {
                    'options': ['percent', 'propability', 'density',
                                'propability density'],
                    'value': self.plot_options.get('histnorm', None),
                    'multi': False
                },
                'trendline': {
                    'options': self.numeric_columns,
                    'value': None,
                    'multi': False
                },
                'dimensions': {
                    'options': self.columns,
                    'value': self.plot_options.get('dimensions', self.columns),
                    'multi': True
                }

            })

    def plot_option_layout(self):
        '''Renders a dropdown widget for each plot option'''
        divs = []
        # The plot type dropdown is handled separate
        divs.append(
            html.Div(
                style=self.style_options_div,
                children=[
                    html.P('Plot type'),
                    dcc.Dropdown(
                        id=f'{self.plot_option_id}-plottype',
                        clearable=False,
                        options=[{'label': i, 'value': i}
                                 for i in self.plots.keys()],
                        value=self.plot_options.get('type', 'scatter')
                    )
                ]
            )
        )
        # Looping through all available plot options
        # and renders a dropdown widget
        for key, arg in self.plot_args.items():
            divs.append(
                html.Div(
                    style=self.style_options_div,
                    id=f'{self.plot_option_id}-div-{key}',
                    children=[
                        html.P(key),
                        dcc.Dropdown(
                            id=f'{self.plot_option_id}-{key}',
                            clearable=False,
                            options=[{'label': i, 'value': i}
                                     for i in arg['options']],
                            value=arg['value'],
                            multi=arg['multi'])
                    ])
            )
        return divs

    @property
    def style_options_div(self):
        '''Style for active plot options'''
        return {
            'display': 'grid',
        }

    @property
    def style_options_div_hidden(self):
        '''Style for hidden plot options'''
        return {
            'display': 'none',
        }

    @property
    def style_page_layout(self):
        '''Simple grid layout for the page'''
        return {} if self.lock else {
            'display': 'grid',
            'align-content': 'space-around',
            'justify-content': 'space-between',
            'grid-template-columns': '1fr 5fr'
        }

    @property
    def style_selectors(self):
        return {'display': 'none'} if self.lock else {}

    @property
    def layout(self):
        return html.Div(children=[
            html.Div(style=self.style_page_layout, children=[
                html.Div(
                    id=self.selector_row,
                    style=self.style_selectors,
                    children=self.plot_option_layout()),
                html.Div(style={'height': '100%'},
                         children=dcc.Graph(id=self.graph_id, config={
                             'responsive': 'true'})
                         )
            ])
        ])

    @property
    def plot_output_callbacks(self):
        '''Creates list of output dependencies for callback
        The outputs are the graph, and the style of the plot options'''
        outputs = []
        outputs.append(
            Output(self.graph_id, 'figure')
        )
        for plot_arg in self.plot_args.keys():
            outputs.append(
                Output(f'{self.plot_option_id}-div-{plot_arg}', 'style')
            )
        return outputs

    @property
    def plot_input_callbacks(self):
        '''Creates list of input dependencies for callback
        The inputs are the plot type and the current value
        for each plot option
        '''
        inputs = []
        inputs.append(
            Input(f'{self.plot_option_id}-plottype', 'value'))
        for plot_arg in self.plot_args.keys():
            inputs.append(Input(f'{self.plot_option_id}-{plot_arg}', 'value'))
        return inputs

    def set_callbacks(self, app):
        @app.callback(self.container_data_output,
                      [self.container_data_requested])
        def _user_download_data(data_requested):
            return WebvizContainer.container_data_compress(
                [{'filename': 'table_plotter.csv',
                  'content': get_data(self.csv_file).to_csv()}]
            ) if data_requested else ''

        @app.callback(
            self.plot_output_callbacks,
            self.plot_input_callbacks)
        def _update_output(*args):
            '''Updates the graph and shows/hides plot options'''
            plot_type = args[0]
            plotfunc = getattr(px._chart_types, plot_type)
            plotargs = {}
            div_style = []
            for name, plot_arg in zip(self.plot_args.keys(), args[1:]):
                if name in self.plots[plot_type]:
                    plotargs[name] = plot_arg
                    div_style.append(self.style_options_div)
                else:
                    div_style.append(self.style_options_div_hidden)
            return (plotfunc(self.data, **plotargs), *div_style)


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def get_data(csv_file) -> pd.DataFrame:
    return pd.read_csv(csv_file, index_col=None)



import dash_html_components as html
from pathlib import Path
from . import WebvizContainer
from ..webviz_assets import webviz_assets


class ExampleAssets(WebvizContainer):

    def __init__(self, app, picture_path: Path):
        self.asset_url = webviz_assets.add(picture_path)

    @property
    def layout(self):
        return html.Img(src=self.asset_url)



from pathlib import Path
import bleach
import markdown
from markdown.util import etree
from markdown.extensions import Extension
from markdown.inlinepatterns import ImageInlineProcessor, IMAGE_LINK_RE
import dash_core_components as html
from . import WebvizContainer
from ..webviz_assets import webviz_assets
from ..webviz_store import webvizstore


class _WebvizMarkdownExtension(Extension):
    def __init__(self, base_path):
        self.base_path = base_path

        super(_WebvizMarkdownExtension, self).__init__()

    def extendMarkdown(self, md):
        md.inlinePatterns['image_link'] = \
            _MarkdownImageProcessor(IMAGE_LINK_RE, md, self.base_path)


class _MarkdownImageProcessor(ImageInlineProcessor):
    def __init__(self, image_link_re, md, base_path):
        self.base_path = base_path

        super(_MarkdownImageProcessor, self).__init__(image_link_re, md)

    def handleMatch(self, match, data):
        image, start, index = super().handleMatch(match, data)

        if image is None or not image.get('title'):
            return image, start, index

        src = image.get('src')
        caption = image.get('title')

        if src.startswith('http'):
            raise ValueError(f'Image path {src} has been given. Only images '
                             'available on the file system can be added.')

        image_path = Path(src)
        if not image_path.is_absolute():
            image_path = (self.base_path / image_path).resolve()

        url = webviz_assets.add(image_path)

        image_style = ''
        for style_prop in image.get('alt').split(','):
            prop, value = style_prop.split('=')
            if prop == 'width':
                image_style += f'width: {value};'
            elif prop == 'height':
                image_style += f'height: {value};'

        if image_style:
            image.set('style', image_style)

        image.set('src', url)
        image.set('class', '_markdown_image')

        container = etree.Element('span')
        container.append(image)

        etree.SubElement(container,
                         'span',
                         attrib={'class': '_markdown_image_caption'}
                         ).text = caption

        return container, start, index


class Markdown(WebvizContainer):
    '''### Include Markdown

_Note:_ The markdown syntax for images has been extended to support
(optionally) providing width and/or height for individual images.
To specify the dimensions write e.g.
```markdown
![width=40%,height=300px](./example_banner.png "Some caption")
```

This container renders and includes the content from a Markdown file. Images
are supported, and should in the markdown file be given as either relative
paths to the markdown file itself, or absolute paths.

* `markdown_file`: Path to the markdown file to render and include. Either
  absolute path or relative to the configuration file.
'''

    ALLOWED_TAGS = [
        'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
        'b', 'i', 'strong', 'em', 'tt',
        'p', 'br', 'span', 'div', 'blockquote', 'code', 'hr',
        'ul', 'ol', 'li', 'dd', 'dt', 'img', 'a', 'sub', 'sup',
        'table', 'thead', 'tbody', 'tr', 'th', 'td'
    ]

    ALLOWED_ATTRIBUTES = {
        '*': ['id', 'class', 'style'],
        'img': ['src', 'alt', 'title', 'style'],
        'a': ['href', 'alt', 'title']
    }

    ALLOWED_STYLES = ['width', 'height']

    def __init__(self, markdown_file: Path):

        self.markdown_file = markdown_file

        self.html = bleach.clean(
                        markdown.markdown(
                            get_path(self.markdown_file).read_text(),
                            extensions=['tables',
                                        'sane_lists',
                                        _WebvizMarkdownExtension(
                                            base_path=markdown_file.parent
                                                                 )]),
                        tags=Markdown.ALLOWED_TAGS,
                        attributes=Markdown.ALLOWED_ATTRIBUTES,
                        styles=Markdown.ALLOWED_STYLES
                                )

    def add_webvizstore(self):
        return [(get_path, [{'path': self.markdown_file}])]

    @property
    def layout(self):
        return html.Markdown(self.html, dangerously_allow_html=True)


@webvizstore
def get_path(path) -> Path:
    return path



import time
import dash
from webviz_config.common_cache import cache
from webviz_config.containers import _table_plotter


def test_table_plotter(dash_duo):

    app = dash.Dash(__name__)
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)

    csv_file = './tests/data/example_data.csv'
    page = _table_plotter.TablePlotter(app, csv_file)
    app.layout = page.layout
    dash_duo.start_server(app)

    # Wait for the app to render(there is probably a better way...)
    time.sleep(5)

    # Checking that no plot options are defined
    assert page.plot_options == {}

    # Checking that the selectors are not hidden
    selector_row = dash_duo.find_element(f'#{page.selector_row}')
    assert selector_row.get_attribute('style') == ''

    # Checking that the correct plot type is initialized
    plot_dd = dash_duo.find_element(f'#{page.plot_option_id}-plottype')
    assert plot_dd.text == 'scatter'

    # Checking that only the relevant options are shown
    for plot_option in page.plot_args.keys():
        plot_option_dd = dash_duo.find_element(
            f'#{page.plot_option_id}-div-{plot_option}')
        if plot_option not in page.plots['scatter']:
            assert plot_option_dd.get_attribute('style') == 'display: none;'
        else:
            assert plot_option_dd.get_attribute('style') == 'display: grid;'

    # Checking that options are initialized correctly
    for option in ['x', 'y']:
        plot_option_dd = dash_duo.find_element(f'#{page.plot_option_id}-{option}')
        assert plot_option_dd.text == 'Well'


def test_initialized_table_plotter(dash_duo):

    app = dash.Dash(__name__)
    app.css.config.serve_locally = True
    app.scripts.config.serve_locally = True
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)

    csv_file = './tests/data/example_data.csv'
    plot_options = dict(
        x='Well',
        y='Initial reservoir pressure (bar)',
        size='Average permeability (D)',
        facet_col='Segment')

    page = _table_plotter.TablePlotter(
        app, csv_file, lock=True, plot_options=plot_options)
    app.layout = page.layout
    dash_duo.start_server(app)

    # Wait for the app to render(there is probably a better way...)


    # Checking that plot options are defined
    assert page.plot_options == plot_options
    assert page.lock

    # Checking that the selectors are hidden
    selector_row = dash_duo.find_element(f'#{page.selector_row}')
    assert selector_row.get_attribute('style') == 'display: none;'



import os
import glob
from webviz_config.themes import WebvizConfigTheme

default_theme = WebvizConfigTheme(theme_name='default')

default_theme.assets = glob.glob(os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    'default_assets',
    '*')
)

default_theme.plotly_layout = {
    'paper_bgcolor': 'rgba(90, 90, 90)',
    'plot_bgcolor': 'rgba(90, 90, 90)',
    'colorway': ['#14213d',
                 '#3a2d58',
                 '#69356a',
                 '#9a3a6f',
                 '#c84367',
                 '#ea5954',
                 '#fe7c37',
                 '#ffa600']
    }



import os
import jinja2
from .themes import installed_themes
from ._config_parser import ConfigParser
from ._certificate import create_certificate, SERVER_KEY_FILENAME, \
                          SERVER_CRT_FILENAME


def write_script(args, build_directory, template_filename, output_filename):

    config_parser = ConfigParser(args.yaml_file)
    configuration = config_parser.configuration

    configuration['port'] = args.port
    configuration['portable'] = True if args.portable else False
    configuration['localhostonly'] = not args.not_only_localhost

    theme = installed_themes[args.theme]
    configuration['csp'] = theme.csp
    configuration['feature_policy'] = theme.feature_policy
    configuration['external_stylesheets'] = theme.external_stylesheets
    configuration['plotly_layout'] = theme.plotly_layout

    configuration['ssl_context'] = '({!r}, {!r})'\
                                   .format(SERVER_CRT_FILENAME,
                                           SERVER_KEY_FILENAME)

    create_certificate(build_directory)

    template_environment = jinja2.Environment(
                 loader=jinja2.PackageLoader('webviz_config', 'templates'),
                 undefined=jinja2.StrictUndefined
                )

    template = template_environment.get_template(template_filename)

    with open(os.path.join(build_directory, output_filename), 'w') as fh:
        fh.write(template.render(configuration))

    return config_parser.assets



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

tests_requires = [
    'chromedriver-binary>=74.0.3729.6.0',
    'pylint~=2.3',
    'pycodestyle~=2.5',
    'selenium~=3.141',
    'mock',
    'pytest-xdist'
]

setup(
    name='webviz-config',
    description='Configuration file support for webviz',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/equinor/webviz-config',
    author='R&T Equinor',
    packages=find_packages(exclude=['tests']),
    package_data={
        'webviz_config': [
            'templates/*',
            'static/*',
            'static/.dockerignore',
            'static/assets/*',
            'themes/default_assets/*'
        ]},
    entry_points={
        'console_scripts': [
           'webviz=webviz_config.command_line:main'
         ],
    },
    install_requires=[
        'dash~=1.1',
        'bleach~=3.1',
        'cryptography~=2.4',
        'dash-auth~=1.3',
        'flask-caching~=1.4',
        'flask-talisman~=0.6',
        'jinja2~=2.10',
        'markdown~=3.0',
        'pandas~=0.24',
        'pyarrow~=0.11',
        'pyyaml~=5.1',
        'webviz-core-components>=0.0.3'
    ],
    tests_require=tests_requires,
    extras_require={'tests': tests_requires},
    setup_requires=['setuptools_scm~=3.2'],
    use_scm_version=True,
    zip_safe=False,
    classifiers=[
     'Programming Language :: Python :: 3',
     'Operating System :: OS Independent',
     'Natural Language :: English',
     'Topic :: Multimedia :: Graphics',
     'Topic :: Scientific/Engineering',
     'Topic :: Scientific/Engineering :: Visualization',
     'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)'
    ]
)



import mock
from pathlib import Path
import dash
from webviz_config.common_cache import cache
from webviz_config.containers import _syntax_highlighter

# mocked functions
get_path = 'webviz_config.containers'\
                '._syntax_highlighter.get_path'


def test_syntax_highlighter(dash_duo):

    app = dash.Dash(__name__)
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)
    code_file = Path('./tests/data/basic_example.yaml')
    with mock.patch(get_path) as mock_path:
        mock_path.return_value = code_file
        page = _syntax_highlighter.SyntaxHighlighter(app, code_file)
        app.layout = page.layout
        dash_duo.start_server(app)
        assert dash_duo.get_logs() == [], "browser console should contain no error"



import pandas as pd
from . import WebvizContainer
from ..webviz_store import webvizstore
from ..common_cache import cache


class ExamplePortable(WebvizContainer):

    def __init__(self, some_number: int):
        self.some_number = some_number

    def add_webvizstore(self):
        return [(input_data_function, [{'some_number': self.some_number}])]

    @property
    def layout(self):
        return str(input_data_function(self.some_number))


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def input_data_function(some_number) -> pd.DataFrame:
    print("This time I'm actually doing the calculation...")
    return pd.DataFrame(data={'col1': [some_number,   some_number*2],
                              'col2': [some_number*3, some_number*4]})



import os
import time
import socket
import shutil
import tempfile
import subprocess
from yaml import YAMLError
from ._config_parser import ParserError
from ._write_script import write_script
from .themes import installed_themes


BUILD_FILENAME = 'webviz_app.py'
STATIC_FOLDER = os.path.join(os.path.dirname(__file__), 'static')


def build_webviz(args):

    if args.theme not in installed_themes:
        raise ValueError(f'Theme `{args.theme}` is not installed.')

    if args.portable is None:
        build_directory = tempfile.mkdtemp()
    else:
        build_directory = args.portable
        os.makedirs(build_directory)

    shutil.copytree(os.path.join(STATIC_FOLDER, 'assets'),
                    os.path.join(build_directory, 'assets'))

    for filename in ['README.md', 'Dockerfile', '.dockerignore']:
        shutil.copy(os.path.join(STATIC_FOLDER, filename), build_directory)

    for asset in installed_themes[args.theme].assets:
        shutil.copy(asset, os.path.join(build_directory, 'assets'))

    try:
        if args.portable:
            print('\033[1m\033[94m'
                  'Extracting, processing and saving requested data to build '
                  'folder such that the webviz instance is portable.'
                  '\033[0m')

            write_script(args, build_directory,
                         'copy_data_template.py.jinja2', 'copy_data.py')

            if subprocess.call(['python3', 'copy_data.py'],
                               cwd=build_directory):
                raise RuntimeError('Something went wrong. This is probably '
                                   'not related to webviz-config, but more '
                                   'likely in one of the  dependencies used '
                                   'by one or more containers. See exact '
                                   'error message and traceback above')

            os.remove(os.path.join(build_directory, 'copy_data.py'))

            print('\033[1m\033[94m'
                  'Finished data extraction. All output saved.'
                  '\033[0m')

        non_default_assets = write_script(args,
                                          build_directory,
                                          'webviz_template.py.jinja2',
                                          BUILD_FILENAME)

        for asset in non_default_assets:
            shutil.copy(asset, os.path.join(build_directory, 'assets'))

        if not args.portable:
            run_webviz(args, build_directory)

    finally:
        if not args.portable:
            print(f'Deleting temporary folder {build_directory}')
            shutil.rmtree(build_directory)


def run_webviz(args, build_directory):

    hostname = socket.getfqdn() if args.not_only_localhost else 'localhost'

    print(' \n\033[92m'
          ' Initializing your webviz application. Go to \n'
          f' https://{hostname}:{args.port} \n'
          ' in order to browse it. The files are hosted from '
          f' {build_directory}\n\n'
          ' To shut down the application, press CTRL+C at any time.'
          ' \033[0m\n')

    app_process = subprocess.Popen(['python3', BUILD_FILENAME],
                                   cwd=build_directory)

    lastmtime = os.path.getmtime(args.yaml_file)

    while app_process.poll() is None:
        time.sleep(1)

        try:
            if lastmtime != os.path.getmtime(args.yaml_file):
                lastmtime = os.path.getmtime(args.yaml_file)
                write_script(args, build_directory,
                             'webviz_template.py.jinja2', BUILD_FILENAME)
                print('\033[1m\033[94m'
                      'Rebuilt webviz dash app from configuration file'
                      '\033[0m')

        except (ParserError, YAMLError) as e:
            print(f'{e} \033[91m Fix the error and save the '
                  'configuration file in order to trigger a new '
                  'rebuild. \033[0m')

        except Exception as e:
            app_process.kill()
            print('\033[91m'
                  'Unexpected error. Killing the webviz dash '
                  'application process.'
                  '\033[0m')
            raise e



import os
from pathlib import Path
import dash_html_components as html
from . import WebvizContainer
from ..webviz_assets import webviz_assets


class BannerImage(WebvizContainer):
    '''### Banner image

This container adds a full width _banner image_, with an optional overlayed
title. Useful on e.g. the front page for introducing a field or project.

* `image`: Path to the picture you want to add. Either absolute path or
  relative to the configuration file.
* `title`: Title which will be overlayed over the banner image.
* `color`: Color to be used for the font.
* `shadow`: Set to `False` if you do not want text shadow for the title.
'''

    TOOLBAR_BUTTONS = []

    def __init__(self, image: Path, title: str = '', color: str = 'white',
                 shadow: bool = True):

        self.image = image
        self.title = title
        self.color = color
        self.shadow = shadow

        self.image_url = webviz_assets.add(image)

    @property
    def layout(self):

        style = {'color': self.color,
                 'background-image': 'url({})'.format(self.image_url)}

        if self.shadow:
            style['text-shadow'] = '0.05em 0.05em 0'

            if self.color == 'white':
                style['text-shadow'] += ' rgba(0, 0, 0, 0.7)'
            else:
                style['text-shadow'] += ' rgba(255, 255, 255, 0.7)'

        return html.Div(self.title, className='_banner_image', style=style)
